
let fs = require('fs');
let path = require('path');

let express = require('express');
let { createProxyMiddleware } = require('http-proxy-middleware');
let axios = require('axios');


const cachePath = "C:/cache";
const codePath = "C:/Users/yangjiandong/lib/cpp";
const dataPath = "C:/data";

function getFileOrRequest(file, target, purl, res) {
    //console.log(file);
    if (fs.existsSync(file)) {
        res.sendFile(file);
        return;
    }
    const filedir = path.dirname(file);
    fs.mkdirSync(filedir, { recursive: true });

    //console.log(purl);
    axios.get(purl, {
        headers: {'Referer': target},
        responseType: 'arraybuffer',
    })
    .then(function (response) {
        // handle success
        //console.log(response);
        fs.writeFile(file, response.data, err => {
            if (err) {
                res.status(500).send('ERROR:'+err);
                return;
            }
            res.sendFile(file);
        });
    })
    .catch(function (error) {
        // handle error
        res.status(500).send('ERROR:'+error);
    });
}

function proxyReqeust(req, res, target, newPath, rewritePath) {
    let file = cachePath + newPath + req.path;
    let purl = target+rewritePath+req.url;
    getFileOrRequest(file, target, purl, res);
}

let app = express();

// Add headers
app.use(function (req, res, next) {
    res.header("Cross-Origin-Embedder-Policy", "require-corp");
    res.header("Cross-Origin-Opener-Policy", "same-origin");
    // Pass to next layer of middleware
    next();
});

app.use('/', express.static(codePath));
app.use('/data', express.static(dataPath));

app.use('/terrain', function (req, res, next) {
    proxyReqeust(req, res, 'https://api.mapbox.com', '/terrain', "/v4/mapbox.terrain-rgb");
});

app.use('/gd', function (req, res, next) {
    let newPath = "/gd/"+req.query.z+"/"+req.query.x +"/"+req.query.y;
    let file = cachePath + newPath + ".png";
    let target = 'http://webst01.is.autonavi.com';
    let purl = target+"/appmaptile"+req.url;
    getFileOrRequest(file, target, purl, res);
});

app.use('/osm', function (req, res, next) {
    proxyReqeust(req, res, 'http://tile-b.openstreetmap.fr', '/osm', "/hot");
});

app.use('/vearth', function (req, res, next) {
    proxyReqeust(req, res, 'http://ecn.t0.tiles.virtualearth.net', '/vearth', "/tiles");
});

app.use('/arc', function (req, res, next) {
    proxyReqeust(req, res, 'http://map.geoq.cn', '/arc', "/ArcGIS/rest/services/ChinaOnlineStreetPurplishBlue/MapServer/tile");
});

app.use('/3dtiles', function (req, res, next) {
    proxyReqeust(req, res, 'http://data.mars3d.cn', '/3dtiles', "/3dtiles");
});

app.listen(3000,()=>{
    console.log("server running at http://localhost:3000")
})



